#ifndef F_CPU
#define F_CPU		16000000
#endif

#define __PROG_TYPES_COMPAT__
#include <avr/pgmspace.h>
#include <avr/io.h>
#include <util/delay.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "gfx.h"

#define MORSE_LUNG		150
#define MORSE_SCURT		50
#define MORSE_PAUZA		50

int16_t lifes = 3;
int16_t player_x = 45;
int16_t player_y = 80;
int16_t board[5][5];
int16_t lines[5][3];
int16_t columns[5][3];
int16_t game_over = 0;
int16_t level = 0;
int16_t to_fill = 0;

/* Generates sounds for the buzzer */
void buzzer_morse(int tip_morse) {
	int i;

	/* Alternating PB0 state at each milisecond */
	for (i = 0; i < tip_morse; ++i) {
		PORTB ^= (1 << PB0);

		_delay_ms(1);
	}
}

void init_pins()
{
	/* Set buttons as input + turn on pull-up resistors */
	DDRA &= ~(1 << PA0);
	PORTA |= (1 << PA0);
	
	DDRA &= ~(1 << PA1);
	PORTA |= (1 << PA1);
	
	DDRA &= ~(1 << PA2);
	PORTA |= (1 << PA2);
	
	DDRA &= ~(1 << PA3);
	PORTA |= (1 << PA3);
	
	DDRA &= ~(1 << PA4);
	PORTA |= (1 << PA4);
	
	/* Set leds as output + turn leds on */
	DDRB |= (1 << PB1);
	PORTB |= (1 << PB1);
	
	DDRB |= (1 << PB2);
	PORTB |= (1 << PB2);
	
	DDRB |= (1 << PB3);
	PORTB |= (1 << PB3);
	
	/* Set buzzer as output + stop it */
	DDRB |= (1 << PB0);
	PORTB &= ~(1 << PB0);
	
	/* Graphic LCD settings */
	init();

	DDRD |= (1 << PD7);
	PORTD |= (1 << PD7);
	
	constructor(_width,_height);
}

void drawOne(int16_t x, int16_t y)
{
	drawFastVLine(x, y - 10, 5, Color565(150, 0, 0));
	drawFastVLine(x, y - 5, 5, Color565(150, 0, 0));
}

void drawTwo(int16_t x, int16_t y)
{
	drawFastHLine(x, y - 10, 5, Color565(150, 0, 0));
	drawFastVLine(x + 5, y - 10, 5, Color565(150, 0, 0));
	drawFastHLine(x, y - 5, 5, Color565(150, 0, 0));
	drawFastVLine(x, y - 5, 5, Color565(150, 0, 0));
	drawFastHLine(x, y, 5, Color565(150, 0, 0));
}

void drawThree(int16_t x, int16_t y)
{
	drawFastHLine(x, y - 10, 5, Color565(150, 0, 0));
	drawFastVLine(x + 5, y - 10, 5, Color565(150, 0, 0));
	drawFastHLine(x, y - 5, 5, Color565(150, 0, 0));
	drawFastVLine(x + 5, y - 5, 5, Color565(150, 0, 0));
	drawFastHLine(x, y, 5, Color565(150, 0, 0));
}

void drawFour(int16_t x, int16_t y)
{
	drawFastVLine(x, y - 10, 5, Color565(150, 0, 0));
	drawFastHLine(x, y - 5, 5, Color565(150, 0, 0));
	drawFastVLine(x + 5, y - 10, 5, Color565(150, 0, 0));
	drawFastVLine(x + 5, y - 5, 5, Color565(150, 0, 0));
}

void drawFive(int16_t x, int16_t y)
{
	drawFastVLine(x, y - 10, 5, Color565(150, 0, 0));
	drawFastHLine(x, y - 10, 5, Color565(150, 0, 0));
	drawFastHLine(x, y - 5, 5, Color565(150, 0, 0));
	drawFastVLine(x + 5, y - 5, 5, Color565(150, 0, 0));
	drawFastHLine(x, y, 5, Color565(150, 0, 0));
}

void drawTable()
{
	int16_t i;
	
	for (i = 0; i < 6; i++) {
		drawFastHLine(45, 155 - 15 * i, 5 * 15, Color565(150, 0, 0));
		drawFastVLine(45 + i * 15, 80, 5 * 15, Color565(150, 0, 0));
	}
}

void init_picture()
{
	int16_t i, j;
	
	srand(level);
	
	for (i = 0; i < 5; i++)
		for (j = 0; j < 5; j++) {
			board[i][j] = rand() % 2;
			
			if (board[i][j] == 1)
				to_fill++;
		}
}

void init_numbers()
{
	int16_t i, j, k;
	
	for (i = 0; i < 5; i++) {
		lines[i][0] = 0;
		lines[i][1] = 0;
		lines[i][2] = 0;
		k = 2;
		
		for (j = 4; j >= 0; j--)
			if (board[i][j] == 1)
				lines[i][k]++;
			else if (lines[i][k] != 0)
					k--;
	}

	for (j = 0; j < 5; j++) {
		columns[j][0] = 0;
		columns[j][1] = 0;
		columns[j][2] = 0;
		k = 2;
		
		for (i = 4; i >= 0; i--)
			if (board[i][j] == 1)
				columns[j][k]++;
			else if (columns[j][k] != 0)
					k--;
	}
}

void drawNumbers()
{
	int16_t i, j, aux;
	
	for (i = 0; i < 5; i++)
		for (j = 0; j < 3; j++) {
			aux = lines[i][j];
			
			switch(aux) {
			case 1:
				drawOne(j * 15 + 7, i * 15 + 95 - 3);
				break;
			case 2:
				drawTwo(j * 15 + 7, i * 15 + 95 - 3);
				break;
			case 3:
				drawThree(j * 15 + 7, i * 15 + 95 - 3);
				break;
			case 4:
				drawFour(j * 15 + 7, i * 15 + 95 - 3);
				break;
			case 5:
				drawFive(j * 15 + 7, i * 15 + 95 - 3);
				break;
			default:
				break;
			}
		}
		
	for (i = 0; i < 5; i++)
		for (j = 0; j < 3; j++) {
			aux = columns[i][j];
			
			switch(aux) {
			case 1:
				drawOne(i * 15 + 45 + 7, j * 15 + 50 - 3);
				break;
			case 2:
				drawTwo(i * 15 + 45 + 7, j * 15 + 50 - 3);
				break;
			case 3:
				drawThree(i * 15 + 45 + 7, j * 15 + 50 - 3);
				break;
			case 4:
				drawFour(i * 15 + 45 + 7, j * 15 + 50 - 3);
				break;
			case 5:
				drawFive(i * 15 + 45 + 7, j * 15 + 50 - 3);
				break;
			default:
				break;
			}
		}
}

void lose_life()
{
	/* Less lifes */
	lifes--;
	
	/* Play buzzer */
	buzzer_morse(MORSE_LUNG);
	PORTB &= ~(1 << PB0);
	_delay_ms(MORSE_PAUZA);
	
	/* Turn off led */
	switch(lifes) {
		case 2:
			PORTB &= ~(1 << PB1);
			break;
		case 1:
			PORTB &= ~(1 << PB2);
			break;
		case 0:
			game_over = 1;
			PORTB &= ~(1 << PB3);
			break;
	}
}

void drawEnvironment()
{
	int16_t i, j;
				
	/* Draw background */
	fillScreen(Color565(0, 255, 0));
	drawTable();
	drawNumbers();
	
	/* Draw completed squares */
	for (i = 0; i < 5; i++)
		for (j = 0; j < 5; j++)
			if (board[i][j] == 2)
				fillRect(45 + j * 15, 80 + i * 15, 15, 15, Color565(255, 255, 255));
		
	/* Draw player */
	fillRect(player_x, player_y, 15, 15, Color565(200, 0, 100));
}
int main(void)
{
	int16_t selected_x, selected_y;
	
	init_pins();
	init_picture();
	init_numbers();

	while(1){
		drawEnvironment();
		
		/* Movement + sanity checks */
		/* Left */
		if ((PINA & (1 << PA0)) == 0) {
			player_x -= 15;
			
			if (player_x < 45)
				player_x += 15;
		}
		
		/* Right */
		if ((PINA & (1 << PA1)) == 0) {
			player_x += 15;
			
			if (player_x > 105)
				player_x -= 15;
		}
		
		/* Up */
		if ((PINA & (1 << PA2)) == 0) {
			player_y -= 15;
			
			if (player_y < 80)
				player_y += 15;
		}
		
		/* Down */
		if ((PINA & (1 << PA3)) == 0) {
			player_y += 15;
			
			if (player_y > 140)
				player_y -= 15;
		}
		
		/* Select square */
		if ((PINA & (1 << PA4)) == 0) {
			selected_x = (player_x - 45) / 15;
			selected_y = (player_y - 80) / 15;
			
			if (board[selected_y][selected_x] == 1) {
				board[selected_y][selected_x] = 2;
				to_fill--;
			}
			else if (board[selected_y][selected_x] == 0)
				lose_life();
		}
		
		if (game_over == 1)
			break;
		
		if (to_fill == 0) {
			level++;
			init_picture();
			init_numbers();
		}
	}
	
	drawEnvironment();
	
	return 0;
}
